@@ -1,5 +1,5 @@
 package Terse;
-our $VERSION = '0.114';
+our $VERSION = '0.115';
 use 5.006;
 use strict;
 use warnings;
@@ -17,7 +17,7 @@ BEGIN {
 	$JSON = JSON->new->utf8->canonical(1)->allow_blessed->convert_blessed;
 	%PRIVATE = (
 		map { $_ => 1 } 
-		qw/new run logger logInfo logError websocket delayed_response content_type raiseError graft pretty serialize DESTROY TO_JSON AUTOLOAD/
+		qw/new run logger logInfo logError websocket delayed_response build_terse content_type raiseError graft pretty serialize DESTROY TO_JSON AUTOLOAD/
 	);
 }
 
@@ -35,6 +35,108 @@ sub new {
 	return bless \%args, $pkg;
 }
 
+sub _build_terse {
+	my ($t) = @_;
+
+	if (! $t->{_application}) {
+		$t->response->raiseError('No application passed to run', 500);
+		return $t->_response($t->response);
+	}
+
+	$t->{websocket} = sub {
+		my ($self, %args) = @_;
+		my $websocket = $t->{_websocket_class}->new($self);
+		if (!ref $websocket) {
+			$args{error}->($t, $websocket);
+			return;
+		}
+		$t->{_delayed_response} = sub {
+			my $responder = shift;
+			$websocket->start($t, \%args, $responder);
+		};
+		return $websocket;
+	} unless $t->{websocket} || !$t->{_websocket_class};
+
+	$t->{delayed_response} = sub {
+		my ($self, $response, $sid, $ct, $status) = @_;
+		$sid ||= $self->sid;
+		$status ||= 200;
+		$ct ||= 'application/json';
+		return $self->{_application}->delayed_response_handle(
+			$self, $response, $sid, $ct, $status
+		) if $self->{_application_has_delayed_response_handler};
+		$self->{_delayed_response} = sub {
+			my $responder = shift;
+			my $res = $self->_build_response($sid, $ct, $status);
+			$res = [splice @{$res->finalize}, 0, 2];
+			my $writer = $responder->($res);
+			$response = eval { $response->($writer); };
+			if ($@ || $self->response->error) {
+				$res->[0] = $self->response->status_code || 500;
+				$self->raiseError($@) if $@;
+				push @{$res}, [$self->response->serialize];
+				return $responder->($res);
+			}
+			elsif ($response) {
+				$writer->write($response->serialize);
+			}
+			$writer->close;
+		};
+		$self;
+	} unless $t->{delayed_response};
+
+	$t->{_application}->build_terse($t) if $t->{_application}->can('build_terse');
+	$t->{_application_has_dispatcher} = !! $t->{_application}->can('dispatch');
+	$t->{_application_has_response_handler} = !! $t->{_application}->can('response_handle');
+	$t->{_application_has_delayed_response_handler} = !! $t->{_application}->can('delayed_response_handle');
+
+	$t->{_build_response} = sub {
+		my ($self, $sid, $content, $status) = @_;
+		my $res = $self->request->new_response($self->response->{status_code} ||= $status);
+		$res->cookies($self->cookies) if $self->cookies;
+		$res->headers($self->headers) if $self->headers;
+		$res->cookies->{sid} = $sid if $sid;
+		$res->content_type($content);
+		return $res;
+	} unless $t->{_build_response};
+
+	$t->{content_type} = sub { 
+		$_[0]->{_content_type} = $_[1] if $_[1];
+		return $_[0]->{_content_type};
+	} unless $t->{content_type};
+
+	$t->{_response} = sub {
+		my ($self, $response_body, $sid, $ct, $status) = @_;
+		return $self->{_application}->response_handle(@_) if $self->{_application_has_response_handler};
+		$ct ||= 'application/json';
+		my $res = $self->{_delayed_response};
+		return $res if ($res); 
+		$res = $self->_build_response($sid, $ct, $status || 200);
+		$res->body($response_body->serialize());
+		return $res->finalize;
+	} unless $t->{_response};
+
+	$t->{_dispatch} = sub {
+		my ($self, $method, @params) = @_;
+		my @out = $self->{_application_has_dispatcher} ? eval {
+			$self->{_application}->dispatch($method, $self, @params)
+		} : eval {
+			unless ($self->{_application}->can($method)) {
+				$self->response->raiseError('Invalid request - ' . $method, 400);
+				return;
+			}
+			$self->{_application}->$method($self, @params);
+		};
+		if ($@) {
+			$self->response->raiseError(['Error while dispatching the request', $@], 400);
+			return;
+		}
+		return @out;
+	} unless $t->{_dispatch};
+	
+	return $t;
+}
+
 sub run {
 	my ($pkg, %args) = @_;
 
@@ -45,75 +147,22 @@ sub run {
 		auth => 'auth',
 		insecure_session => 0,
 		content_type => 'application/json',
-		websocket => 1,
+		request_class => 'Plack::Request',
+		websocket_class => 'Terse::WebSocket',
+		sock => 'psgix.io',
+		stream_check => 'psgi.streaming',
 		%args
 	);
 
-	$j->{websocket} = sub {
-		my ($self, %args) = @_;
-		my $env = $self->request->env;
-		if(!$env->{"psgi.streaming"}){ 
-			return $args{error}->($self, 'no psgi streaming');
-    		}
-		my $websocket = Terse::WebSocket->new($self);
-		$self->{_delayed_response} = sub {
-			my $responder = shift;
-			my @headers = $websocket->headers;
-			my $writer = eval { $responder->([101, [@headers]]); };
-			$args{($@ ? 'error' : 'connect')}->($websocket, $responder, $@);
-			my $ping = 'ping';
-			my $reset_rate = $self->websocket_reset_rate ||= 100000;
-			eval {
-				my $ping_rate = $reset_rate;
-				while (1) {
-					$ping_rate--;
-					my $response;
-					if ($ping_rate < 0) {
-						$ping_rate = $reset_rate;
-						$websocket->send($ping);
-						$response = $websocket->recieve() while($ping_rate-- > 0 && !$response);
-						if (!$response || $response ne 'pong') {
-							last;
-						}
-						$ping_rate = $reset_rate;
-					}
-					$response = $websocket->recieve();
-					if ($response) {
-						if ($response =~ m/^invalid_(length|version|host|required_key)$/) {
-							$args{error}->($websocket, $response, $responder);
-							last;
-						} else {
-							$ping_rate = $reset_rate;
-							$args{recieve}->($websocket, $response, $responder);
-						}
-					}
-				}
-			};
-			$args{error}->($websocket, $responder, $@) if ($@);
-			$args{disconnect}->($websocket, $responder) if $args{disconnect};
-			$responder->([200, []]);
-		};
-		return $websocket;
-	} if $j->{_websocket};
+	$j->_build_terse();
 
-	$j->request = Plack::Request->new($args{plack_env});
+	$j->request = $j->{_request_class}->new($args{plack_env});
 	$j->response = $pkg->new(
 		authenticated => \0,
 		error => \0,
 		errors => [],
 	);
 
-	if (! $j->{_application}) {
-		$j->response->raiseError('No application passed to run', 500);
-		return $j->_response($j->response);
-	}
-
-	$j->{_application}->build_terse($j) if $j->{_application}->can('build_terse');
-
-	$j->{_application_has_dispatcher} = !! $j->{_application}->can('dispatch');
-	$j->{_application_has_response_handler} = !! $j->{_application}->can('response_handle');
-	$j->{_application_has_delayed_response_handler} = !! $j->{_application}->can('delayed_response_handle');
-	
 	my $content_type = $j->request->content_type;
 	if ($content_type && $content_type =~ m/application\/json/) {
 		$j->graft('params', $j->request->raw_body || "{}");
@@ -181,78 +230,6 @@ sub run {
 	return $j->_response($j->response, $j->content_type, $j->sid);
 }
 
-sub _build_response {
-	my ($self, $sid, $content, $status) = @_;
-	my $res = $self->request->new_response($self->response->{status_code} ||= $status);
-	$res->cookies($self->cookies) if $self->cookies;
-	$res->headers($self->headers) if $self->headers;
-	$res->cookies->{sid} = $sid if $sid;
-	$res->content_type($content);
-	return $res;
-}
-
-sub delayed_response {
-	my ($self, $response, $sid, $ct, $status) = @_;
-	$sid ||= $self->sid;
-	$status ||= 200;
-	$ct ||= 'application/json';
-	return $self->{_application}->delayed_response_handle(
-		$self, $response, $sid, $ct, $status
-	) if $self->{_application_has_delayed_response_handler};
-	$self->{_delayed_response} = sub {
-		my $responder = shift;
-		my $res = $self->_build_response($sid, $ct, $status);
-		$res = [splice @{$res->finalize}, 0, 2];
-		my $writer = $responder->($res);
-		$response = eval { $response->($writer); };
-		if ($@ || $self->response->error) {
-			$res->[0] = $self->response->status_code || 500;
-			$self->raiseError($@) if $@;
-			push @{$res}, [$self->response->serialize];
-			return $responder->($res);
-		}
-		elsif ($response) {
-			$writer->write($response->serialize);
-		}
-		$writer->close;
-	};
-	return $self;
-}
-
-sub content_type { 
-	$_[0]->{_content_type} = $_[1] if $_[1];
-	return $_[0]->{_content_type};
-}
-
-sub _response {
-	my ($self, $response_body, $sid, $ct, $status) = @_;
-	return $self->{_application}->response_handle(@_) if $self->{_application_has_response_handler};
-	$ct ||= 'application/json';
-	my $res = $self->{_delayed_response};
-	return $res if ($res); 
-	$res = $self->_build_response($sid, $ct, $status || 200);
-	$res->body($response_body->serialize());
-	return $res->finalize;
-}
-
-sub _dispatch {
-	my ($self, $method, @params) = @_;
-	my @out = $self->{_application_has_dispatcher} ? eval {
-		$self->{_application}->dispatch($method, $self, @params)
-	} : eval {
-		unless ($self->{_application}->can($method)) {
-			$self->response->raiseError('Invalid request - ' . $method, 400);
-			return;
-		}
-		$self->{_application}->$method($self, @params);
-	};
-	if ($@) {
-		$self->response->raiseError(['Error while dispatching the request', $@], 400);
-		return;
-	}
-	return @out;
-}
-
 sub logger {
 	my ($self, $logger) = @_;
 	$self->{_logger} = $logger if ($logger);
@@ -349,7 +326,7 @@ sub DESTROY {}
 
 sub AUTOLOAD : lvalue {
 	my $classname =  ref $_[0];
-	my $validname = '[a-zA-Z][a-zA-Z0-9_]*';
+	my $validname = '[_a-zA-Z][a-zA-Z0-9_]*';
 	our $AUTOLOAD =~ /^${classname}::($validname)$/;
 	my $key = $1;
 	die "illegal key name, must be of $validname form\n$AUTOLOAD" unless $key;
@@ -373,7 +350,7 @@ Terse - Lightweight Web Framework
 
 =head1 VERSION
 
-Version 0.114
+Version 0.115
 
 =cut
 
