@@ -119,14 +119,15 @@ sub PARSE_ATTRIBUTES {
 		elsif ($attribute =~ m/^\s*([^\s\(]+)\s*\(([\s\'\"]*?)(.*)([\s\'\"]*?)\)/) {
 			my $k = lc($1);
 			$attr{$k} = $3;
-			if ($HTTP{$k} || $k eq 'any') {
-				$attr{req} = $3;
-			}	
+			if ($HTTP{$k} || $k =~ m/^(any|websocket|deleayed)$/i) {
+				$attr{req} = $attr{$k};
+			}
 		}
 		else {
 			$attr{lc($attribute)} = 1; 
 		}
 	}
+	$attr{any} = 1 if ($attr{websocket} && scalar keys %attr < 4);
 	return %attr;
 }
 
@@ -205,6 +206,10 @@ sub dispatch {
 	}
 	if ($dispatch->{delayed}) {
 		return $t->delayed_response(sub { $self->$callback($t, @params); $t->response; });
+	} elsif ($dispatch->{websocket}) {
+		$self->websockets ||= {};
+		$self->websockets->{$t->sid->value} = $t->websocket($self->$callback($t, @params), close_delete => 1);
+		return $t;
 	}
 	return $self->$callback($t, @params);
 }
@@ -240,7 +245,7 @@ Terse::Controller - controllers made simple.
 
 =head1 VERSION
 
-Version 0.114
+Version 0.115
 
 =cut
 
@@ -282,6 +287,31 @@ Version 0.114
 		... # 5
 	}
 
+	sub group_chat :websocket {
+		my ($self, $context) = @_;
+		return (
+			connect => sub {
+				$_[0]->send('welcome');
+			},
+			retrieve => sub {
+				my ($websocket, $msg) = @_;
+				# $msg = '{"confrim_sha":"XXX", "message": "Howdy world."}';
+				if ($self->plugin->validate->user_message_sha($context, $websocket->graft('retrieved',$msg))) {
+					$websocket->send($websocket->retrieved->confirm_sha);
+					for my $open (%{ $self->websockets }) {
+						$self->websockets->$open->send($websocket->retrieved->message);
+					}
+				}
+			},
+			error => sub {
+
+			},
+			disconnect => sub {
+
+			}
+		);
+	}
+
 
 	1;
 
@@ -311,6 +341,8 @@ Version 0.114
 
 	GET http://localhost:5000/foo/555/bar #5
 
+	CONNECT ws://localhost:5000/?req=group_chat;
+
 =cut
 
 =head1 AUTHOR
