@@ -1,11 +1,16 @@
 package Terse::WebSocket;
+
 use base 'Terse';
 use MIME::Base64;
+
 sub new {
 	my ($class, $t) = @_;
 	my $self = $class->SUPER::new();
 	my $version = '';
 	my $env =  $t->request->env; 
+	if (!$env->{$t->{_sock}} || !$env->{$t->{_stream_check}}) {
+		return 'Invalid environment no _sock or _stream_check in env';
+	}
 	$self->options = {
 		secret => 'ABABCABC-ABC-ABC-ABCD-ABCABCABCABC',
 		upgrade    => $env->{HTTP_UPGRADE},
@@ -44,6 +49,43 @@ sub new {
 	return $self;
 }
 
+sub start {
+	my ($self, $t, $cbs, $responder) = @_; 
+	my $writer = eval { $responder->([101, [$self->headers]]); };
+	$cbs->{($@ ? 'error' : 'connect')}->($self, $responder, $@);
+	my $reset_rate = $t->websocket_reset_rate ||= 100000;
+	eval {
+		my $ping_rate = $reset_rate;
+		while (1) {
+			$ping_rate--;
+			my $response;
+			if ($ping_rate < 0) {
+				$ping_rate = $reset_rate;
+				$self->send($ping);
+				$response = $self->recieve() while($ping_rate-- > 0 && !$response);
+				if (!$response || $response ne 'pong') {
+					last;
+				}
+				$ping_rate = $reset_rate;
+			}
+			$response = $self->recieve();
+			if ($response) {
+				if ($response =~ m/^invalid_(length|version|host|required_key)$/) {
+					$cbs->{error}->($self, $response, $responder);
+					last;
+				} else {
+					$ping_rate = $reset_rate;
+					$cbs->{recieve}->($self, $response, $responder);
+				}
+			}
+		}
+	};
+	$cbs->{error}->($self, $responder, $@) if ($@);
+	$cbs->{disconnect}->($self, $responder) if $cbs->{disconnect};
+	delete $t->{_application}->websockets->{$t->sid->value} if $cbs->{close_delete};
+	$responder->([200, []]);
+}
+
 sub headers {
 	my ($self) = @_;
 	my $version = $self->version || 'draft-ietf-hybi-10';
@@ -92,33 +134,27 @@ sub headers {
 
 sub send {
 	my ($self, $message) = @_;
-	my $pg = $self->psgix;	
+	my $pg = $self->psgix;
 	my $mask = $self->mask ||= 0;
-	my (@DECODED) = map { ord($_) } split //, $message;
-	my (@ENCODED, @MASK);
-	my $length = scalar @DECODED + 128;
+	my (@ENCODED) = map { ord($_) } split //, $message;
+	my $length = scalar @ENCODED + 128;
 	if ($length > 256 || $mask) {
 		@MASK = map { int(rand(256)) } 0 .. 3;
 		my $i;
-		for (@DECODED) {
-			push @ENCODED, $_ ^ $MASK[$i++ % 4];
-		}
+		$ENCODED[$i++] = $_ ^ $MASK[$i % 4] for (@ENCODED);
 		unshift @ENCODED, @MASK;
 		if ($length > 256) {
-			my $times = int($length / 256) - 1;
+			my $times = int($length / 256);
 			my $excess = $length - (($times * 256) + 128);
-			unshift @ENCODED, $excess;
-			unshift @ENCODED, $times;
-			unshift @ENCODED, 254;
+			unshift @ENCODED, (254, $times, $excess);
 		} else {
 			unshift @ENCODED, $length;
 		}
 	} else {
-		@ENCODED = @DECODED;
 		unshift @ENCODED, $length - 128;
 	}
 	syswrite $pg, join("", map {chr($_)} (129,  @ENCODED));
-	return;
+	return $self;
 }
 
 sub recieve {
@@ -128,14 +164,13 @@ sub recieve {
 	my $len = sysread($pg, $content, 8192);
 	return unless $len;
 	$len = sysread($pg, $content, 8192, length($content)) while $len >= 8192;
-	my $hdr = substr($content, 0, 1);
-	my @bits = split //, unpack("B*", $hdr);
-        $self->fin($bits[0]);
-        $self->rsv([@bits[1 .. 3]]);
-	$content = substr($content, 1, length($content));
+	my @bits = split //, unpack("B*", substr($content, 0, 1));
+        $self->fin = $bits[0];
+        $self->rsv = [@bits[1 .. 3]];
 	my @ENCODED = map { unpack "C", $_ } split //, $content;
+	$self->op = shift @ENCODED;
 	my $length;
-	if (scalar @ENCODED > 126) {
+	if (scalar @ENCODED > 127) {
 		my @length = splice @ENCODED, 0, 3;
 		$length = ((($length[0] + 2) * $length[1]) + $length[2]);
 	} else {
@@ -144,12 +179,7 @@ sub recieve {
 	}
 	my @MASK = splice @ENCODED, 0, 4;
 	return 'invalid_length' if (scalar @ENCODED != $length);
-	my (@DECODED, $i);
-	for (@ENCODED) {
-		push @DECODED, $_ ^ $MASK[$i++ % 4];
-	}
-	$content = join "", map { chr($_) } @DECODED;
-	return $content;
+	return join "", map { chr($ENCODED[$_] ^ $MASK[$_ ^ 4]) } 0 .. scalar @ENCOEDED;
 }
 
 __DONE__;
@@ -162,7 +192,7 @@ Terse::WebSocket - Lightweight WebSockets
 
 =head1 VERSION
 
-Version 0.114
+Version 0.115
 
 =cut
 
@@ -181,23 +211,23 @@ Version 0.114
 	sub chat {
 		my ($self, $t) = @_;
 		$self->webchat->{$t->sid->value} = $t->websocket(
-			connect => {
+			connect => sub {
 				my ($websocket) = @_;
 				$websocket->send('Hello');
 			},
-			recieve => {
+			recieve => sub {
 				my ($websocket, $message) = @_;
 
 				$websocket->send($message); # echo
 			},
-			error => { ... },
-			disconnect => { ... }
+			error => sub { ... },
+			disconnect => sub { ... }
 		);
 	}
 
 	1;
 
-	PERL5LIB=lib:./t/lib plackup -s Starman t/lib/Chat.psgi
+	plackup -s Starman Chat.psgi
 
 	CONNECT ws://localhost:5000?req=chat;
 
