@@ -22,7 +22,7 @@ sub web {
 		},
 		connect => sub {
 			my ($websocket) = @_;
-			$websocket->send('Hello');
+			$websocket->send('Hello ' x 1000);
 		},
 		recieve => sub {
 			my ($websocket) = @_;
